// loadframe.cc -- load data from Frame files in Octave.

// To use this file, your version of Octave must support dynamic
// linking.  To find out if it does, type the command
//
//   x = octave_config_info; x.DEFS
//
// at the Octave prompt.  Support for dynamic linking is included if
// the output contains the string -DWITH_DYNAMIC_LINKING=1.
//
// To compile this file, type the command
//
// mkoctfile --verbose -L[path_to_Framelib] -lFrame loadframe.cc
//
// at the shell prompt.  The script mkoctfile should have been
// installed along with Octave.  Running it will create a file called
// loadframe.oct that can be loaded by Octave.
//
// Enter:
//
// help loadframe
//
// at the Octave prompt to have a description of how loadframe should be used.  
// 
// This file may be used under the terms defined by the GNU 
// General Public License [write to the Free Software Foundation, 
// 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA]
//
// WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
//
// Author: Eric Chassande-Mottin [ecm@obs-nice.fr], Sun Oct 14 00:48:45 CEST 2001
//

#include <octave/config.h>

#include <iostream.h>
#include <string.h>

#include <octave/defun-dld.h>
#include <octave/error.h>
#include <octave/oct-obj.h>
#include <octave/pager.h>
#include <octave/symtab.h>
#include <octave/variables.h>

#include "FrameL.h"


DEFUN_DLD (loadframe, args, ,
  "LOADFRAME: Download an ADC signal in the Octave workspace from a given frame file.\n
Usage:\n
[adc,fs,valid,t0,timegps,unit,slope,bias] = loadframe (fileName,adcName[,nFrames[,first]])\n
Input parameters:\n
fileName: the name of the frame file
adcName:  the name of the ADC signal to be extr.
nFrames:  [opt] the number of frames to be extr. [default=read data as much as possible]
first:    [opt] number of the first frame to be extr. [default=first frame avail.]\n
Output parameters:\n
adc:      the ADC signal
fs:       the sampling frequency
valid:    an index specifying whether the data are OK or not
t0:       the GPS time associated to the first bin in the first extracted frame
timegps:  [string] same thing but human readable format
unit:     physical units of the signal
slope:    slope coef. used to calibrate the signal X
bias:     bias coef. used to calibrate the signal X
")
{
  octave_value_list retval;
 
  /*----------- get input parameters ----------------*/
  int nargin = args.length ();
  
  if (nargin < 2)
    {
      // print_usage("loadframe");
      error("Usage: [adc,fs,valid,t0,timegps,unit,slope,bias] = loadframe(fileName,adcName[,nFrames[,first]])");
      return retval;
    }
  else
    {
      std::ostrstream bufferfName;
      bufferfName << args(0).string_value() << ends;
      char* fileName = bufferfName.str();
      // printf("%s\n",fileName);
    
      std::ostrstream bufferadcName;
      bufferadcName << args(1).string_value() << ends;
      char* adcName = bufferadcName.str();
      // printf("%s\n",adcName);

      //int first=args(2).int_value();
      //int nFrames=args(3).int_value();

      /*----------- open file --------------------------*/
      
      struct FrFile *iFile;  
      iFile = FrFileINew(fileName);
      
      if (iFile == NULL)
	{
	  error("loadframe: cannot open frame file");
	  return retval;
	}
      else
	{

	  /*--------- read the first frame ------------------*/

	  struct FrameH *frame;
	  frame = FrameRead(iFile);
	  
	  if (frame == NULL)
	    {
	      error("loadframe: frame file corrupted\n"); 
	      return retval;
	    }
	  else
	    {

	      if (nargin==4)
		{

		  /*--------- fast forward to the first frame if requested --------------*/

		  int first=(int) args(3).double_value();
		  octave_stdout << "skeeping data from ";
		  octave_stdout << "Run #" << frame->run;
		  octave_stdout << ", frame #" << frame->frame;
		  octave_stdout << " @ GPStime=" << frame->GTimeS << " [...]" << endl;

		  if (frame->frame > first)
		    {
		      error("loadframe: first frame requested not available in given frame file"); 
		      return retval;
		    }
		  else
		    {
		      while(frame->frame!=first)
			{
			  FrameFree(frame);
			  frame = FrameRead(iFile);
			  if (frame == NULL)
			    {
			      error("loadframe: EOF reached and first frame not found"); 
			      return retval;
			    }
			}
		    }

		  octave_stdout << "          [...] to Run #" << frame->run;
		  octave_stdout << ", frame #" << frame->frame;
		  octave_stdout << " @ GPStime=" << frame->GTimeS << endl;

		}

	      /*------ find ADC and read context infos  ---------*/
	      
	      struct FrAdcData *adc;

	      adc = FrAdcDataFind(frame,adcName);

	      if (adc == NULL) 
		{
		  error("loadframe: ADC signal not found in frame");
		  return retval;
		}
	      else
		{
		  double fs = adc->sampleRate;
		  double t0=(double)frame->GTimeS + 1.e-9*(double)frame->GTimeN;
 
		  const time_t tp = frame->GTimeS + 315961200;
		  std::string bufferTime (ctime(&tp),24);
		  std::ostrstream microsec;
		  microsec << frame->GTimeN/1000 << " usec" << ends;
		  std::string timegps = "Starting GPS time: " + bufferTime 
		    +  " (and " + microsec.str() + ")"; 

		  std::string unit; 
		  if(adc->units != NULL)
		    {
		      std::string bufferUnit(adc->units);
		      unit=bufferUnit;
		    }
		  else
		    {
		      std::string bufferUnit("no unit");
		      unit=bufferUnit;
		    }
  
		  double bias = (double) adc->bias;
		  double slope = (double) adc->slope;
		  double valid = (double) adc->dataValid; 

		  retval(1)=fs;
		  retval(2)=valid;
		  retval(3)=t0;
		  retval(4)=timegps;
		  retval(5)= unit; 
		  retval(6)= slope; 
		  retval(7)= bias; 
  
		} // adc ==NULL
	      /* ----------------  read ADC data ----------------*/
	      
	      int index=0, i=0;
	      struct FrVect *vect;

	      int nFrames;
	      if (nargin>2)
		nFrames=(int) args(2).double_value();
	      else
		nFrames=iFile->nFrames;
	      
	      int nData = adc->data->nData;
	      int nTot = nData*nFrames;

	      ColumnVector data(nTot);
	      
	      while (index<nTot)
		{
		  // uncomment to be verbose
		  // octave_stdout << "reading Run #" << frame->run;
		  // octave_stdout << ", frame #" << frame->frame;
		  // octave_stdout << " @ GPStime=" << frame->GTimeS << endl;
		  
		  vect = adc->data;
		  if (vect == NULL) 
		    {
		      error("loadframe: corrupted frame in file");
		      return retval;
		    }
		  else if(vect->type == FR_VECT_C)
		    {for(i=0; i<nData; i++) {data(i+index) = (double) vect->data[i];}}
  		  else if(vect->type == FR_VECT_2S)
		    {for(i=0; i<nData; i++) {data(i+index) = (double) vect->dataS[i];}}
		  else if(vect->type == FR_VECT_4S)
		    {for(i=0; i<nData; i++) {data(i+index) = (double) vect->dataI[i];}}
		  else if(vect->type == FR_VECT_1U)
		    {for(i=0; i<nData; i++) {data(i+index) = (double) vect->dataU[i];}}
		  else if(vect->type == FR_VECT_2U)
		    {for(i=0; i<nData; i++) {data(i+index) = (double) vect->dataUS[i];}}
		  else if(vect->type == FR_VECT_4U)
		    {for(i=0; i<nData; i++) {data(i+index) = (double) vect->dataUI[i];}}
		  else if(vect->type == FR_VECT_4R)
		    {for(i=0; i<nData; i++) {data(i+index) = (double) vect->dataF[i];}}
		  else if(vect->type == FR_VECT_8R)
		    {for(i=0; i<nData; i++) {data(i+index) = (double) vect->dataD[i];}}
		  index += nData;
		  
		  FrameFree(frame);
		  
		  /* ----   read next frame ----- */
		  frame = FrameRead(iFile);
		  if(frame == NULL)
		    {		  
		      error("loadframe: EOF reached before expected");
		      return retval;
		    }
		  adc = FrAdcDataFind(frame,adcName);
		  if (adc == NULL)
		    { 
		      error("loadframe: ADC signal not found in frame");
		      return retval;
		    }
		    
		}
	      
	      /* ------------- Close file and release descriptor ----------*/
	      
	      FrFileIEnd(iFile);    
	      
	      /* ------------- set up and send return values -------------*/
	      
	      retval(0)=data;

	    } // frame == NULL
	} // iFile == NULL
    } // nargin < 3

  return retval;
}









